/*

   ComboDoc  : Combined Documention ( Help + Sample )
   
   Build a sampled documentation set for HMG
   
   Author : Bicahi Esgici; 2013, Oct
   
*/

#include <hmg.ch>

#include "ComboDoc.ch"

PROCEDURE Main()

   PRIVATE cExeName  := ExeName(),;   
           aDocFolds := {},;
           aPrgFolds := {},;
           aNewDirs  := {},;
           aDExclude := {},;
           aDInclude := {},;
           cUnHRFNam := "Excludes.lst"
           
                         // Global Folder Names
   PRIVATE cHMGRoot,;    // HMG Root folder
           cODocRDir,;   // Original DOC root Folder
           cODocDDir,;   // Original DOC data Folder
           cSDocRDir,;   // Sampled Docs root folder
           cSDocDDir     // Sampled Docs data folder
             
   DEFINE WINDOW frmSampledDoc ;
      ROW 0 ;
      COL 0 ;
      WIDTH 550 ;
      HEIGHT 450 ;
      TITLE 'Build sampled doc for HMG' ;
      ON INIT ( Initialize(), BuildSDoc() ) ;
      MAIN
      
      ON KEY ESCAPE ACTION ThisWindow.Release

      DEFINE STATUSBAR FONT 'Verdana' SIZE 8
          STATUSITEM ""
          STATUSITEM ""
      END STATUSBAR

   END WINDOW

   CENTER Window frmSampledDoc

   ACTIVATE Window frmSampledDoc

RETURN // SampledDoc.Main()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

PROCEDURE Initialize()                    // Read .ini file and set global info
  
   LOCAL cIniFName := ChangFExt( ExeName(), "ini" ),;
         nPntr1, nPntr2,;         
         cSysParams,;
         aSysParams     
         
   LOCAL cGFoldLin,;
         c1GVarNam,;
         c1GVarVal

   IF FILE( cIniFName ) 
   
      cSysParams := HB_MEMOREAD( cIniFName )
      
      nPntr1 := AT( "=", cSysParams ) + 1
      nPntr2 := AT( CRLF, SUBSTR( cSysParams, nPntr1 ) )
      cHMGRoot := SUBSTR( cSysParams, nPntr1, nPntr2 ) 
      cHMGRoot := ALLTRIM( LEFT( cHMGRoot, AT( "//", cHMGRoot ) - 1 ) )

      cSysParams := STRTRAN( cSysParams, "<hmgRoot>", cHMGRoot )
      
      aSysParams := HB_ATOKENS( cSysParams, CRLF2 )
      
      aGFoNames := ArryFArry( HB_ATOKENS( aSysParams[ 1 ], CRLF ) )      
      
      FOR EACH cGFoldLin IN  aGFoNames
          c1GVarNam := ALLTRIM( LEFT( cGFoldLin, AT( "=", cGFoldLin ) - 1 ) ) 
          c1GVarVal := ALLTRIM( SUBSTR( cGFoldLin, AT( "=", cGFoldLin ) + 1 ) )
          &c1GVarNam := c1GVarVal
      NEXT cGFoldLin
      
      aDocFolds := ArryFArry( HB_ATOKENS( aSysParams[ 2 ], CRLF ) )
      aPrgFolds := ArryFArry( HB_ATOKENS( aSysParams[ 3 ], CRLF ) )
      
      aDInclude :=  ArryFArry( HB_ATOKENS( aSysParams[ 4 ], CRLF ) )
      
      AEVAL( aDInclude, { | c1, i1 | aDInclude[ i1 ] := HL_List2Arr( c1 ) } )
      
      aNewDirs  := { cSDocRDir, cSDocDDir }
          
   ELSE
      MsgStop( "Couldn't locate .ini file; " + CRLF2 + "Program aborted !", "ERROR !!!" )
   ENDIF   
   
RETURN // Initialize()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.


FUNCTION ArryFArry( aArray )              // Extract an array from an array ( for a special structure )

   LOCAL aRVal := {}

   AEVAL( aArray, { | c1 | IF( !EMPTY( c1 ) .AND. LEFT( ALLTRIM( c1 ), 1 ) # "*",;                              
                               AADD( aRVal, ALLTRIM( IF( "//" $ c1, LEFT( c1, AT( "//", c1 ) - 1 ), c1 ) ) ), ) } )
                              
RETURN aRVal // ArryFArry( )      

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION IsInArray(;                      // Is there this value in this array ?  ( for single dim arrays ONLY )
                   xValu,;   // value to search
                   aArry,;   // array to search in
                   lExEq,;   // exact equality
                   lSubS,;   // substring comparison
                   lIgnC )   // ignore case
                   
   LOCAL lRVal   := .F.,;
         bSearch
         
   HB_Default( @lSubS, .F. )
   HB_Default( @lIgnC, .T. )
   HB_Default( @lExEq, !lSubS )
   
   xValu := IF( lIgnC, UPPER( xValu ), xValu )
   
   IF lExEq
      bSearch := { | x1 | IF( lIgnC, UPPER( x1 ), x1 ) == xValu }
   ELSE
      IF lSubS
         bSearch := { | x1 | xValu $ IF( lIgnC, UPPER( x1 ), x1 ) }
      ELSE
         bSearch := { | x1 | IF( lIgnC, UPPER( x1 ), x1 ) = xValu }
      ENDIF
   ENDIF      

   lRVal := ( ASCAN( aArry, bSearch ) > 0 )
   
RETURN lRVal // IsInArray()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION FNamsList( ;
                    cDirSpec,; 
                    cMask,; 
                    lNest ) 
           
   LOCAL aRVal := {}
   
   HB_Default( @cMask, '*.*' )
   HB_Default( @lNest, .T. )
   
  AEVAL( IF( lNest, HB_DirScan( cDirSpec, cMask ), Directory( cDirSpec + cMask ) ),; 
         { | a1 | AADD( aRVal, cDirSpec + a1[ 1 ] ) } )
          
RETURN aRVal // FNamsList() 

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

PROCEDURE AddArray( ;                      // Combine two arrays
                  aHost,;
                  aGuest )
                  
   AEVAL( aGuest, { | x1 | AADD( aHost, x1 ) } )
   
RETURN // AddArray()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

PROCEDURE MakeUnHRefs( aFList )

   LOCAL aUHRDocs := {},;
         aExclude := {},;
         c1HTMCod,;
         c1FName,;
         c2FName,;
         c1FContent,;
         cToken,;
         nHRef,;
         cText
        
   IF FILE( cUnHRFNam )
   
      aDExclude := HB_ATOKENS( HB_MEMOREAD( cUnHRFNam ), CRLF )      
      
   ELSE
   
      nBegSeco := SECONDS()
      
      WaitWindow( "Preparing to first run; please wait ... ( you have less than 2 minute to take a coffee ;) )" , .T. )
      
      FOR EACH c1FName IN aFList
      
         IF LOWER( FExtFFS( c1FName ) ) == "htm" 
         
            c1HTMCod := FNExFFS( c1FName )
            
            cToken := '<a href="' + c1HTMCod
            
            nHRef := 0
            
            FOR EACH c2FName IN aFList
            
               frmSampledDoc.StatusBar.Item( 1 ) := c2FName
                     
               c1FContent := HB_MEMOREAD( c2FName )
                
               nHRef += IF( cToken $ c1FContent, 1, 0 )
               
               DO EVENTS
               
               frmSampledDoc.StatusBar.Item( 2 ) := PADC( SUBSTR( SECTOTIME( SECONDS() - nBegSeco ), 5 ), 21 ) 
                
            NEXT c2FName
            
            AADD( aUHRDocs, { nHRef, c1FName } )
            
         ENDIF      
         
      NEXT c1FName 
      
      cText := ''
      
      AEVAL( aUHRDocs, { | a1 | IF( ! EMPTY( a1 ) .AND. a1[ 1 ] < 1,; 
                                    ( AADD( aDExclude, a1[ 2 ] ), ;  
                                      cText += a1[ 2 ] + CRLF ), ) } )
                                     
                                     
      cText := LEFT( cText, LEN( cText ) - 2 )
      
      HB_MEMOWRIT( cUnHRFNam, cText )
      
      WaitWindow()
      
   ENDIF FILE( cUnHRFNam )
   
RETURN // MakeUnHRefs()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_List2Arr( ;
                      cString,; 
                      cDelimiter )
             
   LOCAL aRVal
   
   HB_Default( @cDelimiter, "," )
   
   aRVal := HB_ATOKENS( cString, cDelimiter )   
      
   AEVAL( aRVal, { | c1, i1 | aRVal[ i1 ] := ALLTRIM( c1 ) } )
   
RETURN aRVal // HL_List2Arr()      

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

PROCEDURE BuildSDoc()                     // Build HMG Sampled Doc

   LOCAL aDocFList := {},;   // Doc files list 
         aSmpFList := {},;   // Sample files list 
         c1Folder  := '',;
         cHomePage := "hmgdoc.htm",;
         cEndBody  := "</body>"
               
   LOCAL c1SFName,;
         c1SFNamEx,; 
         cSHTMFName,;
         cTHTMFName,;
         c1HTMFCode,;
         lOnlyCopy,;
         aPrgFNames,;
         c1PRGFName,;
         cHTMFText,;
         cHTMFTxt1,;
         cHTMFTxt2,;
         c1PrgHRef,; 
         cPrgHRefs
         
   LOCAL aDocsInfo,;
         aDoc1Info,;
         aSamplSet         

   FILEDELETE( cSDocDDir + '*.*' )
   
   AEVAL( aNewDirs, { | c1 | MakeDir( c1 ) } )
   
   COPY FILE ( cODocRDir + cHomePage  ) TO ( cSDocRDir + cHomePage )

   AEVAL( aDocFolds, { | c1 | AddArray( aDocFList, FNamsList( c1, , .T. ) ) } )

   MakeUnHRefs( aDocFList )
   
   AEVAL( aPrgFolds, { | c1 | AddArray( aSmpFList, FNamsList( c1, "*.prg", .T. ) ) } )

   FOR EACH c1SFName IN aDocFList
   
      lOnlyCopy := .F.

      IF NotAHtmF( c1SFName ) 
         lOnlyCopy := .T.      
      ELSE
      
         IF ExcldHtm( c1SFName ) // excluded 
            lOnlyCopy := .T.
         ELSE
            cSHTMFName := c1SFName                           // Source
            c1HTMFCode := FONmFFS( cSHTMFName )
            
            aDocsInfo := {}
            
            c1HTMFCod1 := c1HTMFCode
            
            aDoc1Info := {}
            
            IF ( nInclude := ASCAN( aDInclude, { | a1 | LOWER( c1HTMFCode ) == LOWER( a1[ 1 ] ) } ) ) > 0 
               cIHTMFCode := aDInclude[ nInclude, 2 ]             

               aSamplSet := HL_ArrSubSet( aSmpFList, { | c1 | UPPER( cIHTMFCode ) $ UPPER( c1 ) } )
                  
               IF !EMPTY( aSamplSet )
                  
                  aDoc1Info := { c1HTMFCode,;
                                 cSHTMFName,;
                                 aSamplSet }
                                 
                  AADD( aDocsInfo, aDoc1Info )
                   
               ENDIF
               
            ENDIF
            
            cHTMFText := HB_MEMOREAD( cSHTMFName ) // <<<
            
            aSamplSet := HL_ArrSubSet( aSmpFList, { | c1 | UPPER( c1HTMFCode ) $ UPPER( c1 ) } )
                  
            IF !EMPTY( aSamplSet )
                 
               aDoc1Info := { c1HTMFCode,;
                              cSHTMFName,;
                              aSamplSet }
                                    
               AADD( aDocsInfo, aDoc1Info )
                     
            ENDIF
               
            IF EMPTY( aDocsInfo )
               lOnlyCopy := .T.
            ELSE                 
               
               FOR EACH aDoc1Info IN aDocsInfo
                  
                  c1HTMFCode := aDoc1Info[ 1 ]
                  cSHTMFName := aDoc1Info[ 2 ]                    
                  cTHTMFName := cSDocDDir + FNExFFS( cSHTMFName ) 
                  aPrgFNames := aDoc1Info[ 3 ]
                                          
                  IF ! EMPTY( aPrgFNames ) 
                  
                     cPrgHRefs := ''
                        
                     FOR EACH c1PRGFName IN aPrgFNames
                     
                        c1PrgHRef := "<p> <a href=" + '"' + c1PRGFName + '"> Sample ' +;
                                     IF( LEN( aPrgFNames ) > 1, '( ' + NTOC( c1PRGFName:__ENUMINDEX() ) + " )", '' ) +;
                                     " ( " + c1PRGFName + " ) </a> </p>"
                                  
                        cPrgHRefs += c1PrgHRef + CRLF
                          
                     NEXT c1PRGFName
                        
                     cHTMFTxt1 := LEFT( cHTMFText, AT( cEndBody, cHTMFText ) - 1 )
                     cHTMFTxt2 := SUBSTR( cHTMFText, AT( cEndBody, cHTMFText ) - 1 )
                     cHTMFText := cHTMFTxt1 + cPrgHRefs + cHTMFTxt2
                        
                  ENDIF ! EMPTY( aPrgFNames )  
                  
                  HB_MEMOWRIT( cTHTMFName, cHTMFText ) 
                  
                  frmSampledDoc.StatusBar.Item( 1 ) := c1SFName
                  
                  DO EVENTS
                  NEXT aDoc1Info
               ENDIF EMPTY( aDocsInfo )
         ENDIF ExcldHtm( 
      ENDIF NotAHtmF
      
      IF lOnlyCopy 
         COPY FILE ( c1SFName ) TO ( cSDocDDir + FNExFFS( c1SFName ) )
      ENDIF
      
   NEXT // cSHTMFName IN aHTMList 

   frmSampledDoc.StatusBar.Item( 1 ) := ''
  
   EXECUTE FILE ( cSDocRDir + cHomePage )
   
QUIT
  
RETURN // BuildSDoc()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

/*

   f.ArrSubSet() : Return a conditional set from an array
   
   Syntax : ArrSubSet( <aArray>  : The source array ( single or double dimension )
                       <bCond>   : A code block to define search condition 
                       [<nElem>] : Element number in <aArray>; if ! NIL, <aArray> is double dim
                       [<lIndx>] : Return indexes only, not values
                     ) --> <aArraySubSet>  : Target array
   
   This function is based upon AExtract() function of SuperLib.
   
   Thanks to author, Gary Prefontaine.

   Bicahi Esgici; Oct 2013
   
*/

FUNCTION HL_ArrSubSet( ;
                   aArray,;
                   bCond,;
                   nElem,;
                   lIndx )
                   
   LOCAL aRVal := {}
   LOCAL xElem 
   
   HB_Default( @lIndx, .F. )
   
   FOR EACH xElem IN aArray 
      
     IF EVAL( bCond, xElem, xElem:__ENUMINDEX() )
       IF HB_ISNIL( nElem ) 
         AADD( aRVal, IF( lIndx, xElem:__ENUMINDEX(), xElem ) )
       ELSE
         AADD( aRVal, IF( lIndx, xElem:__ENUMINDEX(), xElem[ nElem ] ) )
       ENDIF
     ENDIF
     
   NEXT

RETURN aRVal // HL_ArrSubSet()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

